@@ -5,23 +5,13 @@ use strict;
 
 my @properties;
 
-$JSON::PP56::VERSION = '1.01';
+$JSON::PP56::VERSION = '0.11';
 
 BEGIN {
+    *JSON::PP::JSON_encode_ascii = *JSON::PP::_encode_ascii;
+
     sub utf8::is_utf8 {
-        my $len =  length $_[0]; # char length
-        {
-            use bytes; #  byte length;
-            return $len != length $_[0]; # if !=, UTF8-flagged on.
-        }
-    }
-
-    sub utf8::upgrade {
-        ; # noop;
-    }
-
-    sub utf8::downgrade {
-        ; # noop;
+        1; # It is considered that UTF8 flag on for Perl 5.6.
     }
 
     sub utf8::encode (\$) { # UTF8 flag off
@@ -29,9 +19,7 @@ BEGIN {
     }
 
     sub utf8::decode (\$) { # UTF8 flag on
-        if ( _is_valid_utf8(${$_[0]}) ) {
-            ${$_[0]} = pack("U*", unpack("U*", ${$_[0]}));
-        }
+        ${$_[0]} = pack("U*", unpack_emu(${$_[0]}));
     }
 }
 
@@ -43,9 +31,7 @@ unless ($@) {
         eval q| *Unicode::String::utf16be = *Unicode::String::utf16 |;
     }
 
-    *JSON::PP::JSON_encode_ascii   = *_encode_ascii;
-    *JSON::PP::JSON_encode_latin1  = *_encode_latin1;
-    *JSON::PP::JSON_decode_unicode = *JSON::PP::_decode_unicode;
+    *JSON::PP::JSON_encode_ascii = *_encode_ascii;
 
     eval q|
         sub Encode::encode {
@@ -68,26 +54,16 @@ unless ($@) {
 
     $JSON::PP::_ENABLE_UTF16 = 1;
 
-    push @JSON::PP::_properties, 'ascii', 'latin1';
+    push @JSON::PP::_properties, 'ascii';
 }
 else {
-    *JSON::PP::JSON_encode_ascii   = *_noop_encode_ascii;
-    *JSON::PP::JSON_decode_unicode = *_disable_decode_unicode;
+    $JSON::PP::_ENABLE_UTF16 = 0;
 
     eval q| 
         sub JSON::PP::ascii {
             warn "ascii() is disable in Perl5.6x.";
             $_[0]->{ascii} = 0; $_[0];
         }
-
-        sub JSON::PP::latin1 {
-            warn "latin1() is disable in Perl5.6x.";
-            $_[0]->{latin1} = 0; $_[0];
-        }
-
-        sub JSON::PP::get_ascii { 0; }
-
-        sub JSON::PP::get_latin1 { 0; }
     |;
 }
 
@@ -95,6 +71,8 @@ else {
 sub _encode_ascii {
     join('',
         map {
+            chr($_) =~ /[\x00-\x08\x0b\x0e-\x1f]/ ?
+                sprintf('\u%04x', $_) :
             $_ <= 127 ?
                 chr($_) :
             $_ <= 65535 ?
@@ -106,27 +84,9 @@ sub _encode_ascii {
 }
 
 
-sub _encode_latin1 {
-    join('',
-        map {
-            $_ <= 255 ?
-                chr($_) :
-            $_ <= 65535 ?
-                sprintf('\u%04x', $_) :
-                join("", map { '\u' . $_ }
-                        unpack("H4H4", Encode::encode('UTF-16BE', pack("U", $_))));
-        } unpack_emu($_[0])
-    );
-}
-
-
 sub unpack_emu { # for Perl 5.6 unpack warnings
-    return _is_valid_utf8($_[0]) ? unpack('U*', $_[0]) : unpack('C*', $_[0]);
-}
-
-
-sub _is_valid_utf8 {
     my $str = $_[0];
+    my @ret;
     my $is_utf8;
 
     while ($str =~ /(?:
@@ -146,27 +106,28 @@ sub _is_valid_utf8 {
     {
         if (defined $1) {
             $is_utf8 = 1 if (!defined $is_utf8);
+            if ($is_utf8) {
+                push @ret, unpack('U', $1);
+            }
+            else {
+                push @ret, unpack('C*', $1);
+            }
         }
         else {
             $is_utf8 = 0 if (!defined $is_utf8);
+
             if ($is_utf8) { # eventually, not utf8
-                return;
+                return unpack('C*', $str);
             }
+
+            push @ret, unpack('C', $2);
         }
     }
 
-    return $is_utf8;
+    return @ret;
 }
 
 
-sub _noop_encode_ascii {
-    # noop
-}
-
-
-sub _disable_decode_unicode { chr(hex($_[0])); }
-
-
 1;
 __END__
 
